@@ -2,11 +2,15 @@
 Defines classes to solve Linear Equations in one and two variables"""
 
 
-class Symbol():
+import matplotlib.pyplot as plt
+import string
+
+
+class Symbol:
     """Symbol object for linear equations (lowercase letters)
     example: 'x', 'y', 'p', 'q', etc."""
 
-    def __init__(self, symbol):
+    def __init__(self, symbol: str) -> None:
         """initialize instance"""
 
         if not isinstance(symbol, str):
@@ -15,27 +19,25 @@ class Symbol():
         if len(symbol) != 1:
             raise ValueError("Symbol should be char of len() == 1")
 
-        import string
-
         if symbol not in string.ascii_lowercase:
             raise ValueError("Symbol should be lowercase letter")
 
         self.symbol = symbol
 
 
-    def __str__(self, /):
+    def __str__(self, /) -> str:
         """defines str(self)"""
 
         return self.symbol
 
 
-    def __repr__(self, /):
+    def __repr__(self, /) -> str:
         """defines repr(self)"""
 
         return f"Symbol('{self.symbol}')"
 
 
-    def __eq__(self, value, /):
+    def __eq__(self, value: 'Symbol', /) -> bool:
         """return self == value"""
 
         return self.symbol == value.symbol
@@ -44,17 +46,19 @@ class Symbol():
 ################################################################################
 
 
-class LinearEquation1D():
+class LinearEquation1D:
     """objects of type ax + b = 0"""
 
-    def __init__(self, a, b, /, *, sym=Symbol("x")):
+    def __init__(
+            self, a: float, b: float, /, *, sym: Symbol|None = Symbol("x")
+        ) -> None:
         """initialize instance"""
 
         values = [a, b]
 
         for val in a, b:
-            if not isinstance(val, (int, float, complex)):
-                raise TypeError(f"unsupported value: must be 'int', 'float'")
+            if not isinstance(val, int|float):
+                raise TypeError(f"unsupported value: must be 'int' / 'float'")
         if not a:
             raise ValueError("'a' cannot be zero")
 
@@ -69,7 +73,7 @@ class LinearEquation1D():
         self.values, self.sym = tuple(values), sym
 
 
-    def __str__(self, /):
+    def __str__(self, /) -> str:
         """defines str(self)"""
 
         val = list(self.values)
@@ -85,21 +89,22 @@ class LinearEquation1D():
             return f"{sign[0]}{val[0]}{self.sym} = 0"
 
 
-    def __repr__(self, /):
+    def __repr__(self, /) -> str:
         """defines repr(self)"""
 
         return f"LinearEquation1D({self.a}, {self.b}, sym={self.sym !r})"
 
 
-    def __neg__(self, /):
+    def __neg__(self, /) -> 'LinearEquation1D':
         """return -self"""
 
         return LinearEquation1D(*(-val for val in self.values), sym=self.sym)
 
 
-    def __eq__(self, value, /):
+    def __eq__(self, value: 'LinearEquation1D', /) -> bool:
         """return self == value
-        scaled linear equations are considered equal"""
+        note: scaled linear equations are considered equal
+        note: linear equations with different symbols are considered unequal"""
 
         if type(self) is not type(value) or (self.sym != value.sym):
             return False
@@ -109,41 +114,40 @@ class LinearEquation1D():
             return solve1D(self) == solve1D(value)
 
 
-    def __add__(self, value, /):
+    def __add__(self, value: 'LinearEquation1D', /) -> 'LinearEquation1D':
         """return self + value"""
 
         if self.sym != value.sym:
             raise ValueError("Symbol of left and right operands must be same")
-        else:
-            return LinearEquation1D(
-                *(self.values[i] + value.values[i] for i in range(2)),
-                sym=self.sym
-            )
+
+        return LinearEquation1D(
+            *(self.values[i] + value.values[i] for i in range(2)), sym=self.sym
+        )
 
 
-    def __sub__(self, value, /):
+    def __sub__(self, value: 'LinearEquation1D', /) -> 'LinearEquation1D':
         """return self - value"""
 
         return self + (-value)
 
 
-    def __mul__(self, value, /):
+    def __mul__(self, value: float, /) -> 'LinearEquation1D':
         """return self * value
-        value is an int / float to scale the linear equation"""
+        scales the linear equation self by the real number value"""
 
         if not value:
             raise ValueError("can't multiply linear equation by zero")
-        else:
-            return LinearEquation1D(
-                *(self.values[i] * value for i in range(2)), sym=self.sym
-            )
+
+        return LinearEquation1D(
+            *(self.values[i] * value for i in range(2)), sym=self.sym
+        )
 
     __rmul__ = __mul__
 
 
-    def __truediv__(self, value, /):
+    def __truediv__(self, value: float, /) -> 'LinearEquation1D':
         """return self / value
-        value is an int / float to scale the linear equation"""
+        scales the linear equation self by the real number 1/value"""
 
         if not value:
             raise ZeroDivisionError("can't divide linear equation by zero")
@@ -154,17 +158,20 @@ class LinearEquation1D():
 ################################################################################
 
 
-class LinearEquation2D():
+class LinearEquation2D:
     """objects of type ax + by + c = 0"""
 
-    def __init__(self, a, b, c, /, *, sym1=Symbol("x"), sym2=Symbol("y")):
+    def __init__(
+            self, a: float, b: float, c: float, /, *,
+            sym1: Symbol|None = Symbol("x"), sym2: Symbol|None = Symbol("y")
+        ) -> None:
         """initialize instance"""
 
         values = [a, b, c]
 
         for i in range(3):
-            if not isinstance(values[i], (int, float, complex)):
-                raise TypeError(f"unsupported value: must be 'int', 'float'")
+            if not isinstance(values[i], int|float):
+                raise TypeError(f"unsupported value: must be 'int' / 'float'")
 
             if i != 2 and not values[i]:
                 raise ValueError(f"'{'b' if i else 'a'}' cannot be zero")
@@ -182,7 +189,7 @@ class LinearEquation2D():
         self.sym1, self.sym2, self.symbols = sym1, sym2, (sym1, sym2)
 
 
-    def __str__(self, /):
+    def __str__(self, /) -> str:
         """defines str(self)"""
 
         val = list(self.values)
@@ -198,7 +205,7 @@ class LinearEquation2D():
             return f"{val[0]}{self.sym1} {val[1]}{self.sym2} = 0"
 
 
-    def __repr__(self, /):
+    def __repr__(self, /) -> str:
         """defines repr(self)"""
 
         return " ".join((
@@ -207,7 +214,7 @@ class LinearEquation2D():
         ))
 
 
-    def __neg__(self, /):
+    def __neg__(self, /) -> str:
         """return -self"""
 
         return LinearEquation2D(
@@ -215,9 +222,10 @@ class LinearEquation2D():
         )
 
 
-    def __eq__(self, value, /):
+    def __eq__(self, value: 'LinearEquation2D', /) -> bool:
         """return self == value
-        scaled linear equations are considered equal"""
+        note: scaled linear equations are considered equal
+        note: linear equations with different symbols are considered unequal"""
 
         if type(self) is not type(value) or (self.symbols != value.symbols):
             return False
@@ -231,7 +239,9 @@ class LinearEquation2D():
             ) == 1 and (self.c == value.c == 0)
 
 
-    def __add__(self, value, /):
+    def __add__(
+            self, value: 'LinearEquation2D', /
+        ) -> 'LinearEquation2D'|LinearEquation1D:
         """return self + value"""
 
         if self.symbols != value.symbols:
@@ -256,30 +266,31 @@ class LinearEquation2D():
                 )
 
 
-    def __sub__(self, value, /):
+    def __sub__(
+            self, value: 'LinearEquation2D', /
+        ) -> 'LinearEquation2D'|LinearEquation1D:
         """return self - value"""
 
         return self + (-value)
 
 
-    def __mul__(self, value, /):
+    def __mul__(self, value: float, /) -> 'LinearEquation2D':
         """return self * value
-        value is an int / float to scale the linear equation"""
+        scales the linear equation self by the real number value"""
 
         if not value:
             raise ValueError("can't multiply linear equation by zero")
-        else:
-            return LinearEquation2D(
-                *(self.values[i] * value for i in range(3)),
-                sym1=self.sym1, sym2=self.sym2
-            )
+
+        return LinearEquation2D(
+            *(self.values[i] * value for i in range(3)), sym1=self.sym1, sym2=self.sym2
+        )
 
     __rmul__ = __mul__
 
 
-    def __truediv__(self, value, /):
+    def __truediv__(self, value: float, /) -> 'LinearEquation2D':
         """return self / value
-        value is an int / float to scale the linear equation"""
+        scales the linear equation self by the real number 1/value"""
 
         if not value:
             raise ZeroDivisionError("can't divide linear equation by zero")
@@ -290,9 +301,8 @@ class LinearEquation2D():
 ################################################################################
 
 
-def solve1D(eqn: LinearEquation1D):
-    """solve 1 linear equation in 1 variable
-    return answer as float"""
+def solve1D(eqn: LinearEquation1D) -> float:
+    """solve 1 linear equation in 1 variable"""
 
     if not isinstance(eqn, LinearEquation1D):
         raise TypeError("solve1D expects LinearEquation1D as arg")
@@ -300,7 +310,7 @@ def solve1D(eqn: LinearEquation1D):
         return float(-eqn.b / eqn.a)
 
 
-def consistency(eqn1: LinearEquation2D, eqn2: LinearEquation2D):
+def consistency(eqn1: LinearEquation2D, eqn2: LinearEquation2D) -> int:
     """return number of solutions (0, 1 or 2) to refer to
     the consistency of 2 LinearEquation2D objects:
         • 0 -> parallel
@@ -319,9 +329,8 @@ def consistency(eqn1: LinearEquation2D, eqn2: LinearEquation2D):
         return 0
 
 
-def solve2D(eqn1: LinearEquation2D, eqn2: LinearEquation2D):
+def solve2D(eqn1: LinearEquation2D, eqn2: LinearEquation2D) -> tuple[float, float]|None:
     """solve 2 linear equations in 2 variables
-    return answer as tuple of floats
     return None if no solutions exist"""
 
     for eqn in eqn1, eqn2:
@@ -329,7 +338,7 @@ def solve2D(eqn1: LinearEquation2D, eqn2: LinearEquation2D):
             raise TypeError("solve2D expects LinearEquation2D as args")
 
     if not consistency(eqn1, eqn2):
-        return None, None
+        return None
 
     if consistency(eqn1, eqn2) == 2:
         return float("inf"), float("inf")
@@ -341,14 +350,12 @@ def solve2D(eqn1: LinearEquation2D, eqn2: LinearEquation2D):
     return x, y
 
 
-def satisfies(eqn: LinearEquation2D, /, x, y):
+def satisfies(eqn: LinearEquation2D, /, x: float, y: float) -> bool:
     """returns True if given x and y values satisfy the given LinearEquation2D,
-    False otherwise. x and y can be int / float."""
+    False otherwise"""
 
     if not isinstance(eqn, LinearEquation2D):
-        raise TypeError(
-            "invalid argument type for eqn: must be LinearEquation2D"
-        )
+        raise TypeError("invalid argument type for eqn: must be LinearEquation2D")
 
     for num in x, y:
         if not isinstance(num, (int, float)):
@@ -357,18 +364,27 @@ def satisfies(eqn: LinearEquation2D, /, x, y):
     return not x*eqn.a + y*eqn.b + eqn.c
 
 
-def graph(eqn1: LinearEquation2D, eqn2: LinearEquation2D):
+def graph(eqn1: LinearEquation2D, eqn2: LinearEquation2D) -> None:
     """plots two linear equations (and their soluion, if any) on a graph"""
 
-    import matplotlib.pyplot as plt
-
     X, Y = solve2D(eqn1, eqn2)
     plt.figure(figsize=(6, 5))
 
-    y1, y2 = -(eqn1.c / eqn1.b), -(eqn2.c / eqn2.b)
-    s1, s2 = -(eqn1.a / eqn1.b), -(eqn2.a / eqn2.b)
-    l1 = f"{eqn1.a}x {'+' if eqn1.b > 0 else '-'} {abs(eqn1.b)}y = {-eqn1.c}"
-    l2 = f"{eqn2.a}x {'+' if eqn2.b > 0 else '-'} {abs(eqn2.b)}y = {-eqn2.c}"
+    if eqn1.a < 0:
+        a1, b1, c1 = -eqn1.a, -eqn1.b, -eqn1.c
+    else:
+        a1, b1, c1 = eqn1.a, eqn1.b, eqn1.c
+
+    if eqn2.a < 0:
+        a2, b2, c2 = -eqn2.a, -eqn2.b, -eqn2.c
+    else:
+        a2, b2, c2 = eqn2.a, eqn2.b, eqn2.c
+
+    y1, y2 = -(c1 / b1), -(c2 / b2)
+    s1, s2 = -(a1 / b1), -(a2 / b2)
+
+    l1 = f"{a1}x {'+' if b1 > 0 else '-'} {abs(b1)}y = {-c1}"
+    l2 = f"{a2}x {'+' if b2 > 0 else '-'} {abs(b2)}y = {-c2}"
 
     plt.axhline(y=0, color="black", linewidth=3)
     plt.axvline(x=0, color="black", linewidth=3)
@@ -380,4 +396,43 @@ def graph(eqn1: LinearEquation2D, eqn2: LinearEquation2D):
 
     plt.grid(True)
     plt.legend()
+    plt.show()
+
+
+def graph_many(*equations: LinearEquation2D, show_legend: bool|None = True) -> None:
+    """plots all linear equations (and their soluions, if any) on the same graph
+    show_legend is a boolean which displays the legend on the plot if True
+    note: more than two arguments are required for graph_many()"""
+
+    if len(equations) <= 2:
+        raise ValueError("more than two arguments required for 'graph_many'")
+
+    colors = ["red", "orange", "green", "blue", "purple"] * len(equations)
+    solutions = []
+    for eqn1 in equations:
+        for eqn2 in equations:
+            if (eqn1 != eqn2 and solve2D(eqn2, eqn1) not in solutions):
+                solutions.append(solve2D(eqn1, eqn2))
+
+    plt.figure(figsize=(6, 5))
+    plt.axhline(y=0, color="black", linewidth=3)
+    plt.axvline(x=0, color="black", linewidth=3)
+
+    for i, eqn in enumerate(equations):
+        a, b, c = eqn.a, eqn.b, eqn.c
+        if a < 0:
+            a, b, c = -a, -b, -c
+        plt.axline(
+            (0, -(c / b)), slope=-(a / b), color=colors[i], linewidth=2,
+            label=f"{a}x {'+' if b > 0 else '-'} {abs(b)}y = {-c}",
+        )
+
+    for x, y in set(solutions):
+        if x not in {None, float("inf")}:
+            plt.plot(x, y, color="black", marker="o", label=f"({x}, {y})")
+
+    if show_legend:
+        plt.legend()
+
+    plt.grid(True)
     plt.show()
\ No newline at end of file
